###########################################################################
#
# dbutil::gdbm -- utility functions for writing to gdbm databases
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 2009
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package dbutil::gdbm;

use strict;

use dbutil::gdbmtxtgz;

# -----------------------------------------------------------------------------
#   GDBM IMPLEMENTATION
# -----------------------------------------------------------------------------

sub open_infodb_write_handle
{
  my $infodb_file_path = shift(@_);
  my $opt_append = shift(@_);

  my $txt2db_exe = &util::filename_cat($ENV{'GSDLHOME'},"bin",$ENV{'GSDLOS'}, "txt2db" . &util::get_os_exe());
  my $infodb_file_handle = undef;
  my $cmd = "\"$txt2db_exe\"";
  if ((defined $opt_append) && ($opt_append eq "append")) {
      $cmd .= " -append";
  }
  $cmd .= " \"$infodb_file_path\"";

  if (!-e "$txt2db_exe")
  {
      print STDERR "Error: Unable to find $txt2db_exe\n";
      return undef;
  }

  if(!open($infodb_file_handle, "| $cmd"))
  {

      print STDERR "Error: Failed to open pipe to $cmd\n";
      print STDERR "       $!\n";
      return undef;
  }

  binmode($infodb_file_handle,":utf8");

  return $infodb_file_handle;
}



sub close_infodb_write_handle
{
  my $infodb_handle = shift(@_);

  close($infodb_handle);
}


sub get_infodb_file_path
{
  my $collection_name = shift(@_);
  my $infodb_directory_path = shift(@_);

  my $infodb_file_extension = ".gdb";
  my $infodb_file_name = &util::get_dirsep_tail($collection_name) . $infodb_file_extension;
  return &util::filename_cat($infodb_directory_path, $infodb_file_name);
}


sub read_infodb_file
{
  my $infodb_file_path = shift(@_);
  my $infodb_map = shift(@_);

  open (PIPEIN, "db2txt \"$infodb_file_path\" |") 
      || die "couldn't open pipe from db2txt \$infodb_file_path\"\n";

  binmode(PIPEIN,":utf8");

  my $infodb_line = "";
  my $infodb_key = "";
  my $infodb_value = "";
  while (defined ($infodb_line = <PIPEIN>))
  {
    if ($infodb_line =~ /^\[([^\]]+)\]$/)
    {
      $infodb_key = $1;
    }
    elsif ($infodb_line =~ /^-{70}$/)
    {
      $infodb_map->{$infodb_key} = $infodb_value;
      $infodb_key = "";
      $infodb_value = "";
    }
    else
    {
      $infodb_value .= $infodb_line;
    }
  }

  close (PIPEIN);
}

sub read_infodb_keys
{
  my $infodb_file_path = shift(@_);
  my $infodb_map = shift(@_);

  open (PIPEIN, "gdbmkeys \"$infodb_file_path\" |") 
      || die "couldn't open pipe from gdbmkeys \$infodb_file_path\"\n";

  binmode(PIPEIN,":utf8");

  my $infodb_line = "";
  my $infodb_key = "";
  my $infodb_value = "";
  while (defined ($infodb_line = <PIPEIN>))
  {
      chomp $infodb_line; # remove end of line 

      $infodb_map->{$infodb_line} = 1;
  }

  close (PIPEIN);
}

sub write_infodb_entry
{
  # With infodb_handle already set up, works the same as _gdbm_txtgz version
  &dbutil::gdbmtxtgz::write_infodb_entry(@_);
}

sub write_infodb_rawentry
{
  # With infodb_handle already set up, works the same as _gdbm_txtgz version
  &dbutil::gdbmtxtgz::write_infodb_rawentry(@_);
}


sub delete_infodb_entry
{
  # With infodb_handle already set up, works the same as _gdbm_txtgz version
  &dbutil::gdbmtxtgz::delete_infodb_entry(@_);
}



1;
